name: {{ name }}
title: "{{ display_name }}"
version: 1.0.0
description: "{{ description }}"
automation_type: webhook
kind: workflow
base_price: 1

input_schema:
  type: object
  required:
    - event_type
    - payload
  properties:
    event_type:
      type: string
      description: "Type of webhook event"
    payload:
      type: object
      description: "Webhook payload data"
    headers:
      type: object
      description: "Request headers"
    timestamp:
      type: string
      format: date-time
      description: "Event timestamp"

output_schema:
  type: object
  properties:
    processed:
      type: boolean
      description: "Whether event was processed"
    event_id:
      type: string
      description: "Unique event identifier"
    actions_taken:
      type: array
      description: "List of actions performed"
    response:
      type: object
      description: "Response data for webhook caller"

required_credentials: []
optional_credentials: []

category: integration
tags:
  - webhook
  - event-driven
  - integration

author_name: "{{ author_name }}"

estimated_duration_seconds: 30

limits:
  max_duration_seconds: 60
  max_memory_mb: 512
  max_llm_calls: 50
  max_http_requests: 20
