name: {{ name }}
title: "{{ display_name }}"
version: 1.0.0
description: "{{ description }}"
automation_type: standard
kind: workflow
base_price: 1

input_schema:
  type: object
  required:
    - data
  properties:
    data:
      type: string
      description: "Input data to process"

output_schema:
  type: object
  properties:
    result:
      type: string
      description: "Processed result"
    success:
      type: boolean
      description: "Whether processing succeeded"

required_credentials: []
optional_credentials: []

category: general
tags:
  - automation
  - basic

author_name: "{{ author_name }}"

estimated_duration_seconds: 60

limits:
  max_duration_seconds: 300
  max_memory_mb: 512
  max_llm_calls: 50
  max_http_requests: 20
