"""
{{ display_name }} - State definition.

This module defines the state schema used throughout the automation workflow.
"""

from typing import Annotated, Optional

from torivers_sdk.automation import BaseState, merge_dict


class {{ class_name }}State(BaseState):
    """State for {{ display_name }} automation.

    Attributes:
        data: Input data to process.
        result: Processed result.
        success: Whether processing succeeded.
    """

    # Input fields
    data: Optional[str] = None

    # Processing fields
    result: Optional[str] = None
    success: bool = False

    # Accumulated data (uses merge strategy)
    metadata: Annotated[dict, merge_dict] = {}
